//  File SerialCamera.pde
//  Demo code for using seeeduino or Arduino board to cature jpg format 
//  picturefrom seeed serial camera and save it into sd card. Push a 
//  button with led on, meaning start to capture the pic, and with led 
//  off meaning the capture has finished.

//  For more details about the product please check http://www.seeedstudio.com/depot/

//  Copyright (c) 2010 seeed technology inc.
//  Author: Icing Chang
//  Version: 1.0
//  Time: Oct 27, 2010
//  Changing records:
//    
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


#include <Fat16.h>
#include <Fat16util.h> // use functions to print strings from flash memory

SdCard card;
Fat16 file;

#define CAPTURE_LED_PIN  5
#define CAPTURE_BUTTON_PIN  6

#define CMD_LEN 11
#define PIC_BUF_LEN 64
#define SHIFT_BIT 6

#define sendCmd(cmd) {int t = 0; for(t = 0; t < CMD_LEN; t++) Serial.print(cmd[t]);}

//for the cmd info,please refer to the datasheet of serial camera
char cmdVGA[] = {
  0x55,0x41,0x52,0x54,0xCA,0x00,0x00,0x00,0x00,0x00,0x00};
char cmdQVGA[] = {
  0x55,0x41,0x52,0x54,0xCA,0x00,0x00,0x00,0xFF,0x00,0x00};
char cmdGetDat[] = {
  0x55,0x41,0x52,0x54,0xC7,0x00,0x00,0x00,0x00,0x00,0x00};


//byte picBuf[PIC_BUF_LEN];//buffer to temperally save pic data from camera, not use yet
int picNameNum = 0;

void readCamSaveToFile(Fat16 &file, int toBeReadLen);

/******************************************************
 * Note: As the max speed of soft serial is 57600,lower
 * than the speed of seial camera 115200. We have to use 
 * the hard serial on seeeduino or arduino board. And you
 * should not open the Serial terminal when the code is 
 * running. There is also a demo code for mega, which uses
 * the hardware seiral1.
 *******************************************************/
void setup()
{
  Serial.begin(115200);
  pinMode(CAPTURE_LED_PIN,OUTPUT);
  pinMode(CAPTURE_BUTTON_PIN,INPUT);
  // initialize the SD card
  if (!card.init()) 
  {
    return;//error
  }
  // initialize a FAT16 volume
  if (!Fat16::init(&card))
  {
    return;//error
  }

  for(int i = 0; i< 5;i++){//flash led 5 times
    digitalWrite(CAPTURE_LED_PIN,HIGH);
    delay(100);
    digitalWrite(CAPTURE_LED_PIN,LOW);
    delay(100);
  }
}

void loop()
{
  //add an button control here
  while(digitalRead(CAPTURE_BUTTON_PIN) == LOW);//wait until the button is pushed
  digitalWrite(CAPTURE_LED_PIN,HIGH);//led on, meaning starts to cature

  //send get pic size request
  Serial.flush();
  unsigned long picTotalLen = 0;
  int i = 0;
  for(i = 0; i < CMD_LEN; i++){
    Serial.print(cmdVGA[i]);//you can change here to capture QVGA pic by using cmdQVGA[]
  }

  //get pic size response
  while(Serial.available() < 3);
  picTotalLen |= Serial.read()<<16;
  picTotalLen |= Serial.read()<<8;
  picTotalLen |= Serial.read();


  //set the name of the pic to be saved
  char picName[] = "pic00.jpg";
  picName[3] = picNameNum/10 + '0';
  picName[4] = picNameNum%10 + '0';

  //creat and open file
  // O_CREAT - create the file if it does not exist
  // O_EXCL - fail if the file exists
  // O_WRITE - open for write
  if (!file.open(picName, O_CREAT | O_WRITE)){
    return;
  }
  if (!file.isOpen()){
    return;
  }

  //read pic data from camera and write to file in SD card
  unsigned long count = picTotalLen >> SHIFT_BIT;//how many times we have to read data, with reading PIC_BUF_LEN length every time 
  int tail = picTotalLen & (PIC_BUF_LEN - 1);//the reset part of the pic data that is no longer than PIC_BUF_LEN
  unsigned long addr = 0;// start from 0, and then add PIC_BUF_LEN every time we read pic data
  cmdGetDat[5] = addr >> 0;
  cmdGetDat[6] = addr >> 0;
  cmdGetDat[7] = addr >> 0;

  cmdGetDat[10] = PIC_BUF_LEN;

  for(int i = 0; i < count; i++){//get and save count*PIC_BUF_LEN data
    sendCmd(cmdGetDat);
    readCamSaveToFile(file, PIC_BUF_LEN);
    addr += PIC_BUF_LEN;
    cmdGetDat[5] = addr >> 16;
    cmdGetDat[6] = addr >> 8;
    cmdGetDat[7] = addr;
  }

  cmdGetDat[10] = tail;//get reset of the pic data
  sendCmd(cmdGetDat);
  readCamSaveToFile(file, tail);

  // close file and force write of all data to the SD card
  if(!file.close()){
    return;//error
  }
  picNameNum++;//incease the pic number, so that the continuous pic name will changes

  digitalWrite(CAPTURE_LED_PIN,LOW); //led off,meaning the capture succeeded.
}

//read the sepecific length of pic data from serial camera
//and save them to the file on the SD card
void readCamSaveToFile(Fat16 &file, int toBeReadLen)
{
  int readLen = 0;
  while(readLen < toBeReadLen){
    if(Serial.available() > 0){
      byte picDat = Serial.read();
      file.write(&picDat,1);
      readLen++;
    }
  }
}











